var Controllers = angular.module('SocialQuiz.Controllers', []);

Controllers.controller('RootCtrl', function ($rootScope, $localstorage, $translate, Logger) {
    var categories_default = [
        {id: 1, name: 'sport', enabled: true},
        {id: 2, name: 'fashion', enabled: true},
        {id: 3, name: 'media', enabled: true},
        {id: 4, name: 'music', enabled: true},
        {id: 5, name: 'politics', enabled: true},
        {id: 6, name: 'cars', enabled: true},
        {id: 7, name: 'business', enabled: true},
        {id: 8, name: 'food', enabled: true},
        {id: 9, name: 'games', enabled: true},
        {id: 10, name: 'technology', enabled: true}
    ];

    $rootScope.language = navigator.language || navigator.userLanguage;

    if ($rootScope.language.toLowerCase().indexOf('de') >= 0) {
        $rootScope.language = 'de';
    } else {
        $rootScope.language = 'en';
    }

    //$rootScope.language = 'en';

    $translate.use($rootScope.language);

    Logger.log($rootScope.language);


    $rootScope.categories = $localstorage.getObject('selected_categories_3', categories_default);
    $rootScope.category_count = 0;

});


Controllers.controller('ChatsCtrl', function($scope, Chats) {
    $scope.chats = Chats.all();
    $scope.remove = function(chat) {
        Chats.remove(chat);
    }
});

Controllers.controller('DashCtrl', function ($scope, $rootScope, $ionicPopover, $interval, $localstorage, GameMakerGM1, ReviewAsk) {



    $ionicPopover.fromTemplateUrl('categorie-popover.html', {
        scope: $scope
    }).then(function(popover) {
        $scope.popover = popover;
    });

    $scope.selectCategories = function ($event) {
        $scope.popover.show($event);
    };


    $scope.$on('popover.hidden', function() {
        $localstorage.setObject('selected_categories_3', $rootScope.categories);
    });

    $scope.$on('$ionicView.enter', function () {
        if (window.analytics) {
            window.analytics.trackView('Play');
        }
        ReviewAsk.checkAskForReview();
    });

    $scope.updateCategoryCount = GameMakerGM1.updateCategoryCount;

    $scope.updateCategoryCount();


});

Controllers.controller('ChatDetailCtrl', function($scope, $stateParams, Chats) {
    $scope.chat = Chats.get($stateParams.chatId);
});

Controllers.controller('AccountCtrl', function($scope) {
    $scope.settings = {
        enableFriends: true
    };
});

Controllers.controller('GM1Ctrl', function($scope, $rootScope, $ionicNavBarDelegate, $interval, $location, $translate, GameMakerGM1, Logger) {

    $translate('game.headline').then(function (string) {
        $scope.header = string;
    });

    $scope.service = {};

    GameMakerGM1.runWithScope($scope.service);

    $scope.remaining_seconds = 20;

    var game_stop = false;
    var ad_counter = 5;

    var interval_call = function () {
        $scope.remaining_seconds -= 1;
        if ($scope.remaining_seconds <= 0) {
            $interval.cancel(stop);
            $scope.game_stop = true;
            $scope.show_timer = false;
            GameMakerGM1.gameFailed();
        }
    };

    var stop = $interval(interval_call, 1000);


    $scope.pageClick = function (selection) {
        if (!game_stop) {
            game_stop = true;
            $interval.cancel(stop);
            Logger.log(selection);
            GameMakerGM1.validateSelection(selection);
        }
    };


    $scope.nextRound = function () {
        GameMakerGM1.startRound();
        $scope.remaining_seconds = 20;
        game_stop = false;
        GameMakerGM1.showTimer();
        stop = $interval(interval_call, 1000);

    };

    $scope.showAd = function () {
        $scope.restart();
    };

    $scope.restart = function () {
        GameMakerGM1.startRound();
        $scope.remaining_seconds = 20;
        game_stop = false;
        GameMakerGM1.showTimer();
        stop = $interval(interval_call, 1000);
    };

    $scope.showResult = function () {
        $location.path('/tab/dash/gm1/result/6/10/60');
    };

    $scope.quit = function () {
        if(AdMob) {
            AdMob.showInterstitial();
            $location.path('/tab/dash');
        } else {
            $location.path('/tab/dash');
        }

    };

    $scope.share = function () {
        GameMakerGM1.share();
    };

    $scope.$on('$destroy', function() {
        $interval.cancel(stop);
        Logger.log('GM1Ctrl $destroy');
    });


    $rootScope.$on('$locationChangeSuccess', function(){
        //$scope.$destroy();
    });

});

Controllers.controller('ResultScreenCtrl', function ($scope, $stateParams) {

    $scope.category_counter = $stateParams.catCount;
    $scope.round_counter = $stateParams.roundCount;
    $scope.points = $stateParams.points;


});


Controllers.controller('TabBarCtrl', function ($scope, $rootScope, $state, Gamecenter) {

    $scope.showHighscore = Gamecenter.showHighscore;

    var tabbar_hide_states = ['tab.dash-gm1', 'tab.dash-gm1-result', 'tab.dash'];

    $rootScope.$on('$ionicView.beforeEnter', function() {

        $rootScope.hideTabs = false;

        if (tabbar_hide_states.indexOf($state.current.name) >= 0) {
            $rootScope.hideTabs = true;
        }

    });

});


Controllers.controller('HighscoreCtrl', function ($scope, $rootScope, Facebook) {


    $scope.$on('$ionicView.beforeEnter', function(){

    });


    $scope.getHighscore = function () {
        Facebook.getFacebookScores(function (data) { console.log(data) }, function () {});
    };


    $scope.setHighscore = function () {
        Facebook.setFacebookScore(10, function (data) { console.log(data) }, function () {})
    }



});
