var Services = angular.module('SocialQuiz.Services', []);

Services.factory('Chats', function() {
    // Might use a resource here that returns a JSON array

    // Some fake testing data
    var chats = [{
        id: 0,
        name: 'Ben Sparrow',
        lastText: 'You on your way?',
        face: 'https://pbs.twimg.com/profile_images/514549811765211136/9SgAuHeY.png'
    }, {
        id: 1,
        name: 'Max Lynx',
        lastText: 'Hey, it\'s me',
        face: 'https://avatars3.githubusercontent.com/u/11214?v=3&s=460'
    }, {
        id: 2,
        name: 'Andrew Jostlin',
        lastText: 'Did you get the ice cream?',
        face: 'https://pbs.twimg.com/profile_images/491274378181488640/Tti0fFVJ.jpeg'
    }, {
        id: 3,
        name: 'Adam Bradleyson',
        lastText: 'I should buy a boat',
        face: 'https://pbs.twimg.com/profile_images/479090794058379264/84TKj_qa.jpeg'
    }, {
        id: 4,
        name: 'Perry Governor',
        lastText: 'Look at my mukluks!',
        face: 'https://pbs.twimg.com/profile_images/491995398135767040/ie2Z_V6e.jpeg'
    }];

    return {
        all: function() {
            return chats;
        },
        remove: function(chat) {
            chats.splice(chats.indexOf(chat), 1);
        },
        get: function(chatId) {
            for (var i = 0; i < chats.length; i++) {
                if (chats[i].id === parseInt(chatId)) {
                    return chats[i];
                }
            }
            return null;
        }
    };
});



Services.factory('$localstorage', ['$window', function($window) {
    return {
        set: function(key, value) {
            $window.localStorage[key] = value;
        },
        get: function(key, defaultValue) {
            return $window.localStorage[key] || defaultValue;
        },
        setInt: function(key, value) {
            $window.localStorage[key] = parseInt(value);
        },
        getInt: function(key, defaultValue) {
            return parseInt($window.localStorage[key]) || parseInt(defaultValue);
        },
        setObject: function(key, value) {
            $window.localStorage[key] = JSON.stringify(value);
        },
        getObject: function(key, defaultObject) {
            var storage = $window.localStorage[key];
            if (!storage) {
                return defaultObject;
            } else {
                return JSON.parse(storage || '{}');
            }

        }
    }
}]);


Services.factory('GameData', ['$http', '$q', '$rootScope', 'Logger', function ($http, $q, $rootScope, Logger) {

    var db = null;
    var database = null;

    var scripted_data = {
        'de': [
            [[1, 7146002927, 'Hamburg', 'Hamburg', 'DE', 'PERSON', 5, 4, 781000],[1, 10733764354, 'Berlin', 'Berlin', 'DE', 'PERSON', 5, 4, 1677895]],
            [[1, 103852469653050, 'House-music', 'House Music', 'DE', 'PERSON', 5, 4, 4831972], [1, 108566315834100, 'Rock-Music', 'Rock Music', 'DE', 'PERSON', 5, 4, 22978839]],
            [[1, 6815841748, 'barackobama', 'Barack Obama', 'DE', 'PERSON', 5, 4, 42000000], [1, 120065791373626, 'Pope-Francis', 'Pope Francis', 'DE', 'PERSOM', 5, 4, 1770151]]
        ],
        'en': [
            [[1, 108424279189115, 'New-York-New-York', 'New York', 'DE', 'PERSON', 5, 4, 14528208],[1, 110884905606108, 'Sydney-Australia', 'Sydney', 'DE', 'PERSON', 5, 4, 1023833]],
            [[1, 103852469653050, 'House-music', 'House Music', 'DE', 'PERSON', 5, 4, 4831972], [1, 108566315834100, 'Rock-Music', 'Rock Music', 'DE', 'PERSON', 5, 4, 22978839]],
            [[1, 6815841748, 'barackobama', 'Barack Obama', 'DE', 'PERSON', 5, 4, 42000000], [1, 120065791373626, 'Pope-Francis', 'Pope Francis', 'DE', 'PERSOM', 5, 4, 1770151]]
        ]
    };

    var first_game = [
        ['shakira', 'katyperry'],
        ['youtube', 'facebook'],
        ['ladygaga', 'pink'],
        ['rihanna', 'deadmau5'],
        ['snickers', 'CocaColaDE'],
        ['TaylorSwift', 'britneyspears'],
        ['MileyCyrus', 'beyonce'],
        ['McDonalds', 'burgerking'],
        ['50cent', 'JayZ'],
        ['MeganFox', 'JenniferLawrence'],
        ['TheSimpsons', 'FamilyGuy'],
        ['netflix', 'twitterinc'],
        ['candycrushsaga', 'angrybirds']
    ];

    var shuffle = function (o) {
        for(var j, x, i = o.length; i; j = Math.floor(Math.random() * i), x = o[--i], o[i] = o[j], o[j] = x);
        return o;
    };

    var promise = $http.get('js/data.sqlite', {responseType: 'arraybuffer'}).success(function (data) {
        var uInt8Array = new Uint8Array(data);
        db = new SQL.Database(uInt8Array);
        database = db.exec('SELECT * FROM facebook_pages')[0];
        database.values = database.values.filter(function (entry) {
            return entry[database.columns.indexOf('country')].toLowerCase() == $rootScope.language || entry[database.columns.indexOf('country')].toLowerCase() == 'us';

            //return entry[database.columns.indexOf('country')].toLowerCase() == $rootScope.language;
        });
        shuffle(database.values);
    });

    var map_data = function (data) {
        return {
            id: data[database.columns.indexOf('id')],
            object_id: data[database.columns.indexOf('object_id')],
            object_key: data[database.columns.indexOf('object_key')],
            name: data[database.columns.indexOf('name')],
            likes: parseInt(data[database.columns.indexOf('likes')]),
            category_display: parseInt(data[database.columns.indexOf('category_display')]),
            country: data[database.columns.indexOf('country')].toLowerCase()
        }
    };

    var selected_category_ids;


    return {
        promise:promise,

        convertMapData: function (data) {
            return map_data(data);
        },

        getFirstGameData: function (round) {
            var object_keys = first_game[round];
            var pages;
            if (object_keys) {
                pages = database.values.filter(function (value) {
                    if (object_keys.indexOf(value[database.columns.indexOf('object_key')]) > -1) {
                        return true;
                    }
                });
                if (pages.length >= 2) {
                    return [map_data(pages[0]), map_data(pages[1])];
                }
            }
            return null;
        },

        getScriptedData: function (round,entry) {
            var object = scripted_data[$rootScope.language][round][entry];
            return map_data(object);
        },

        setCategories: function (categories) {
            selected_category_ids = [];
            for (var i = 0; i < categories.length; i++) {
                if (categories[i].enabled) {
                    selected_category_ids.push(categories[i].id);
                }
            }
        },

        test: function () {
            Logger.log(db.exec("SELECT * FROM facebook_pages ORDER BY RANDOM() LIMIT 1;"));
        },

        calcRange: function (likes, min, max) {
            var minRange = likes / 100 * min;
            var maxRange = likes / 100 * max;

            return [minRange, maxRange]
        },

        getRandomPage: function (config) {
            if (!config.pool) {
                config.pool = database.values;
            }

            if (config.minLikes) {
                config.pool = config.pool.filter(
                    function (value) {
                        return parseInt(value[database.columns.indexOf('likes')]) > config.minLikes && (selected_category_ids.indexOf(parseInt(value[database.columns.indexOf('category_display')])) >= 0);
                    }
                );
            }

            var sql_result = config.pool[Math.floor(Math.random()*config.pool.length)];
            if (sql_result) {
                return map_data(sql_result)
            }
            return null
        },

        getPageInRange: function (currentPage, minRange, maxRange) {
            Logger.log('Ranges: ', minRange, maxRange);
            var possible_pages;

            var country_limit = currentPage.country != 'us';

            var min_likes = currentPage.likes / 100 * minRange;
            var max_likes = currentPage.likes / 100 * maxRange;

            var min_likes_more = currentPage.likes + min_likes;
            var max_likes_more = currentPage.likes + max_likes;

            var min_likes_less = currentPage.likes - min_likes;
            var max_likes_less = currentPage.likes - max_likes;

            if (max_likes_less < 0) {
                min_likes_less += max_likes_less * -1;
                max_likes_less = 0;
            }

            Logger.log(currentPage.name, currentPage.likes, min_likes_more, max_likes_more);
            Logger.log(currentPage.name, currentPage.likes, min_likes_less, max_likes_less);

            var filters = [
                function (value) {
                    var likes = parseInt(value[database.columns.indexOf('likes')]);
                    if (country_limit && value[database.columns.indexOf('country')].toLowerCase() != currentPage.country) {
                        return false;
                    }
                    if (selected_category_ids.indexOf(parseInt(value[database.columns.indexOf('category_display')])) <= -1) {
                        return false;
                    }
                    if (value[database.columns.indexOf('object_id')] == currentPage.object_id) {
                        return false;
                    }
                    return likes > min_likes_more && likes < max_likes_more;
                },

                function (value) {
                    var likes = parseInt(value[database.columns.indexOf('likes')]);
                    if (country_limit && value[database.columns.indexOf('country')].toLowerCase() != currentPage.country) {
                        return false;
                    }
                    if (selected_category_ids.indexOf(parseInt(value[database.columns.indexOf('category_display')])) <= -1) {
                        return false;
                    }
                    if (value[database.columns.indexOf('object_id')] == currentPage.object_id) {
                        return false;
                    }
                    return likes > max_likes_less && likes < min_likes_less;
                }
            ];

            possible_pages = database.values.filter(filters[Math.floor(Math.random()*filters.length)].bind(this));

            if (!possible_pages.length) {
                Logger.log('Still no page in range');
                return this.getPageInRange(currentPage, minRange, maxRange + 10);
            } else {
                return this.getRandomPage({pool: possible_pages});
            }

        }
    }

}]);


Services.factory('GameMakerGM1', ['$interval', '$ionicNavBarDelegate', '$location', '$localstorage','$rootScope', '$translate', 'GameData', 'Gamecenter', 'Logger', 'ReviewAsk', 'Facebook',
    function($interval, $ionicNavBarDelegate, $location, $localstorage, $rootScope, $translate, gamedata, Gamecenter, Logger, ReviewAsk, Facebook) {

        var scope, start_new_game, first_game;


        this.runWithScope = function (_scope) {
            scope = _scope;
            scope.points = 0;
            scope.round_counter = -1;
            scope.highscore = $localstorage.get('gm1-highscore', 0);
            start_new_game = false;
            first_game = $localstorage.get('is_first_game', 'true');
            $localstorage.set('is_first_game', 'false');
            gamedata.setCategories($rootScope.categories);
            ReviewAsk.updateCounter();
            this.updateCategoryCount();
            this.showTimer();
            this.nextRound();
            if (window.plugins && window.plugins.appsFlyer) {
                window.plugins.appsFlyer.sendTrackingWithEvent('game_start', '');
            }

        };

        this.getMinRange = function () {
            var min_range = 800 - (scope.round_counter * 100);
            if (min_range < 0) {
                min_range = 0;
            }
            return min_range
        };

        this.getMaxRange = function () {
            var max_range = 1600 - (scope.round_counter * 200);
            if (max_range < 30) {
                max_range = 30;
            }
            return max_range
        };


        this.nextRound = function () {

            if (start_new_game) {
                scope.points = 0;
                scope.round_counter = -1;
                start_new_game = false;

                this.updatePointsHeader();
                this.showTimer();
            }

            scope.round_counter += 1;

            if (!SCRIPTED_DATA) {

                if (first_game == 'true') {

                    var gameplay_data = gamedata.getFirstGameData(scope.round_counter);
                    if (gameplay_data) {
                        Logger.log(gameplay_data);
                        scope.pageOne = gameplay_data[0];
                        scope.pageTwo = gameplay_data[1];
                    } else {
                        scope.pageOne = gamedata.getRandomPage({minLikes: 500000});
                        scope.pageTwo = gamedata.getPageInRange(scope.pageOne, this.getMinRange(), this.getMaxRange());
                    }
                } else {
                    scope.pageOne = gamedata.getRandomPage({minLikes: 500000});
                    scope.pageTwo = gamedata.getPageInRange(scope.pageOne, this.getMinRange(), this.getMaxRange());
                }


            } else {
                scope.pageOne = gamedata.getScriptedData(scope.round_counter, 0);
                scope.pageTwo = gamedata.getScriptedData(scope.round_counter, 1);
            }

            $('.presentation').hide();

        };

        this.getFirstGameData = function () {

        };


        this.validateSelection = function (_selection) {
            var selection, other;

            if (_selection == 'pageOne') {
                selection = scope.pageOne;
                other = scope.pageTwo;
            } else {
                selection = scope.pageTwo;
                other = scope.pageOne;
            }

            if (selection.likes >= other.likes) {
                selection.resultcss = 'result score correct';
                other.resultcss = 'result';
                scope.points += 1 * $rootScope.category_count;
                scope.won_page = selection;
                scope.lose_page = other;
                this.showNextButton();
            } else {
                scope.won_page = other;
                scope.lose_page = selection;
                selection.resultcss = 'result wrong';
                other.resultcss = 'result score';
                this.gameFailed();
            }

            var selecion_like_display = numeral(selection.likes).format('0 a');
            var other_like_display = numeral(other.likes).format('0 a');

            if (selecion_like_display.toString() == other_like_display.toString()) {
                selecion_like_display = numeral(selection.likes).format('0.00 a');
                other_like_display = numeral(other.likes).format('0.00 a');
            }

            selection.like_display = selecion_like_display;
            other.like_display = other_like_display;

            $('.presentation').fadeIn('slow');

        };


        this.showTimer = function () {
            $('#resultcard').hide();
            $('#game-footer').removeClass('showNext');
            $('#game-footer').removeClass('showResult');
            $('#game-footer').addClass('showTimer');
            this.showTitleHeader();
        };

        this.showNextButton = function () {
            $('#timer-bar').stop();
            $('#resultcard').hide();
            $('#game-footer').addClass('showNext');
            $('#game-footer').removeClass('showResult');
            $('#game-footer').removeClass('showTimer');
            this.updatePointsHeader();
        };

        this.showResultButtons = function () {
            $('#timer-bar').stop();
            $('#resultcard').show();
            $('#game-footer').removeClass('showNext');
            $('#game-footer').addClass('showResult');
            $('#game-footer').removeClass('showTimer');
            this.updatePointsHeader();
        };

        this.updatePointsHeader = function () {
            $translate('game.headline_points', {
                points: scope.points
            }).then(function (string) {
                $ionicNavBarDelegate.title(string);
            });
        };

        this.showTitleHeader = function () {
            $translate('game.headline').then(function (string) {
                $ionicNavBarDelegate.title(string);
            });
        };

        this.gameFailed = function () {
            start_new_game = true;
            this.showResultButtons();
            Gamecenter.submitScore(scope.points, 'gm1');
            if (scope.points > scope.highscore) {
                scope.highscore = scope.points;
                $localstorage.set('gm1-highscore', scope.highscore);
            }
            if (window.analytics) {
                window.analytics.trackEvent('Game', 'Finish', 'Rounds', scope.round_counter);
                window.analytics.trackEvent('Game', 'Finish', 'Points', scope.points);
                window.analytics.trackEvent('Game', 'Finish', 'Categories', $rootScope.category_count);
            }

            if (window.plugins && window.plugins.appsFlyer) {
                window.plugins.appsFlyer.sendTrackingWithEvent('game_finished', '');
            }

            if (first_game) {
                first_game = false;
            }



        };

        this.restart = function () {

        };

        this.updateCategoryCount = function () {
            var selected = $rootScope.categories.filter(function (entry) {
                return entry.enabled
            });
            $rootScope.category_count = selected.length;
        };

        this.share = function (cb) {

            var translate_key;
            var options;


            if (start_new_game) {
                translate_key = 'lose';
            } else {
                translate_key = 'win';
            }

            $translate(['share.' + translate_key + '.title', 'share.' + translate_key + '.headline', 'share.' + translate_key + '.desc'], {name1: scope.won_page.name, name2: scope.lose_page.name}).then(function (result) {
                options = {
                    method: "feed",
                    name: result['share.' + translate_key + '.title'],
                    link: "http://m.onelink.me/6462762d",
                    caption: result['share.' + translate_key + '.headline'],
                    description: result['share.' + translate_key + '.desc']
                };

                Logger.log(options);

                Facebook.share(options);
            });


        };



        return {
            runWithScope: this.runWithScope.bind(this),
            validateSelection: this.validateSelection.bind(this),
            startRound: this.nextRound.bind(this),
            showTimer: this.showTimer.bind(this),
            showNextButton: this.showNextButton.bind(this),
            gameFailed: this.gameFailed.bind(this),
            updateCategoryCount: this.updateCategoryCount.bind(this),
            share: this.share.bind(this)
        }
    }]);


Services.factory('Gamecenter', ['$localstorage', 'Logger', function ($localstorage, Logger) {

    var successCallback = function (data) {
    };

    var errorCallback = function (data) {
    };


    var functions = {
        showHighscore: function () {

            if (window.analytics) {
                window.analytics.trackView('Highscore');
            }

            if (ionic.Platform.isAndroid()) {

                googleplaygame.isSignedIn(function (result) {

                    if (!result.isSignedIn) {
                        googleplaygame.auth(function () {

                            functions.submitScore($localstorage.get('gm1-highscore', 0), 'gm1');
                            googleplaygame.showLeaderboard({ leaderboardId: 'CgkIjtez9P8BEAIQBw' });
                            if (window.plugins && window.plugins.appsFlyer) {
                                window.plugins.appsFlyer.sendTrackingWithEvent('show_highscore', '');
                            }


                        }.bind(this), function (data) {
                            Logger.log(data);
                        });
                    } else {
                        googleplaygame.showLeaderboard({ leaderboardId: 'CgkIjtez9P8BEAIQBw' });

                        if (window.plugins && window.plugins.appsFlyer) {
                            window.plugins.appsFlyer.sendTrackingWithEvent('show_highscore', '');
                        }

                    }

                }.bind(this));

            } else if (ionic.Platform.isIOS() && typeof gamecenter !== 'undefined') {

                gamecenter.showLeaderboard(successCallback, errorCallback, { leaderboardId: 'socialquiz.highscore' });


            }


        },
        submitScore: function (score, board_id) {
            var boards = {
                gm1: {
                    ios: 'socialquiz.highscore',
                    android: 'CgkIjtez9P8BEAIQBw'
                }
            };

            var request = {
                score: parseInt(score),
                leaderboardId: board_id
            };


            if (ionic.Platform.isAndroid()) {

                googleplaygame.isSignedIn(function (result) {
                    if (result.isSignedIn) {
                        request.leaderboardId = boards[board_id].android;
                        googleplaygame.submitScore(request);
                    }
                });

            }else if (ionic.Platform.isIOS() && typeof gamecenter !== 'undefined') {
                request.leaderboardId = boards[board_id].ios;
                gamecenter.submitScore(successCallback, errorCallback, request);
            }
        }
    };

    return functions;

}]);


Services.factory('Logger', [function () {

    var empty_console = {
        log: function () {},
        error: function () {}
    };

    if (DEBUG) {
        return console;
    } else {
        return empty_console;
    }

}]);

Services.factory('ReviewAsk', ['$rootScope', '$localstorage', '$translate', 'Logger', function ($rootScope, $localstorage, $translate, Logger) {

    var did_review = $localstorage.get('didReview', false);
    var rate_counter = $localstorage.getInt('rateCounter', 0);
    var _strings = {};

    $translate('review.headline').then(function (string) { _strings.headline = string });
    $translate('review.text').then(function (string) { _strings.text = string });
    $translate('review.button.rate').then(function (string) { _strings.rate = string });
    $translate('review.button.later').then(function (string) { _strings.later = string });
    $translate('review.button.no').then(function (string) { _strings.no = string });

    var open_review_dialog = function () {

        if (navigator.notification) {
            navigator.notification.confirm(
                _strings.text,
                function(button) {
                    // yes = 1, no = 2, later = 3
                    if (button == '1') {    // Rate Now
                        if (ionic.Platform.isIOS()) {

                            if (ionic.Platform.version() >= 7) {
                                window.open('itms-apps://itunes.apple.com/app/id986385596');
                            } else {
                                window.open('itms-apps://itunes.apple.com/WebObjects/MZStore.woa/wa/viewContentsUserReviews?type=Purple+Software&id=986385596');
                            }

                        } else if (ionic.Platform.isAndroid()) {
                            window.open('market://details?id=de.farbflut.socialquizlive');
                        }
                        did_review = true;
                        $localstorage.set('didReview', true);
                    } else if (button == '2') {
                        rate_counter = 0;
                        $localstorage.set('rateCounter', 0);
                    } else if (button == '3') { // No
                        did_review = true;
                        $localstorage.set('didReview', true);
                    }
                }, _strings.headline, [_strings.rate, _strings.later, _strings.no]);
        } else {
            Logger.log('navigator.notification not installed');
        }

    };

    return {
        updateCounter: function () {
            rate_counter += 1;
            $localstorage.setInt('rateCounter', rate_counter);
            Logger.log('updateCounter:', rate_counter);
        },

        checkAskForReview: function () {
            if (RATE_COUNTER < rate_counter && !did_review) {
               open_review_dialog();
            }
            Logger.log('checkAskForReview:', RATE_COUNTER, rate_counter);
        }
    };

}]);


Services.factory('Facebook', ['Logger', function (Logger) {

    var permissions = ['public_profile', 'user_friends'];
    var highscore_example = {
        data: [{
            application: {
                id: '1123340567692173',
                name: 'LikeDuell',
                namespace: 'likeduell'
            },
            score: 1,
            user: {
                id: '1386679701659089',
                name: 'Susan Amieabafiage Romanescu'
            }
        }]
    };

    this.isEnabled = function () {
        return typeof facebookConnectPlugin !== 'undefined';
    };

    this.share = function (share_obj) {
        if (this.isEnabled()) {

            this.login(function () {
                facebookConnectPlugin.showDialog(share_obj, function (data) {
                    if (window.analytics) {
                        window.analytics.trackEvent('Game', 'Social', 'Shared');
                    }
                    if (window.plugins && window.plugins.appsFlyer) {
                        window.plugins.appsFlyer.sendTrackingWithEvent('social_shared', '');
                    }


                }, function (data) { Logger.log(data) });
            }, function (data) {
                Logger.log(data);
            });

        } else {
            Logger.log('facebookConnectPlugin disabled');
        }

    };

    this.login = function (success_cb, error_cb) {
        if (this.isEnabled()) {

            facebookConnectPlugin.getLoginStatus(function (response) {

                if (response && response.status == 'connected') {
                    success_cb();
                } else {
                    facebookConnectPlugin.login(permissions, success_cb, error_cb);
                }

            }.bind(this), function () {

                facebookConnectPlugin.login(permissions, success_cb, error_cb);

            }.bind(this));

        }
    };

    this.isLoggedIn = function (success_cb, error_cb) {
        if (this.isEnabled()) {
            facebookConnectPlugin.getLoginStatus(success_cb, error_cb);
        } else {
            error_cb();
        }
    };

    this.getFacebookScores = function (success_cb, error_cb) {



        if (this.isEnabled()) {

            this.login(function () {
                facebookConnectPlugin.api('/me/scores', [],'get', {},
                    function (data) {
                        console.log(data);
                        success_cb(data);
                    },
                    function (data) {
                        console.log(data);
                    });
            }.bind(this));
        }
    };

    this.setFacebookScore = function (score, success_cb, error_cb) {


        if (this.isEnabled()) {

            this.login(function () {
                facebookConnectPlugin.api('/me/scores', ['publish_actions'],'post', { score: score.toString() },
                    function (data) {
                        console.log(data);
                        success_cb(data);
                    },
                    function (data) {
                        console.log(data);
                    });
            }.bind(this));

        }

    };


    return {
        share: this.share.bind(this),
        getFacebookScores: this.getFacebookScores.bind(this),
        setFacebookScore: this.setFacebookScore.bind(this)
    }

}]);












